searchData={"content_type":"text/markdown","items":[{"doc":"Documentation for `EctoEncryptedId`.","ref":"EctoEncryptedId.html","title":"EctoEncryptedId","type":"module"},{"doc":"Helper function to delegate to Phoenix.Param implementations for Ecto schemas.\nReturns encrypted version of the id.","ref":"EctoEncryptedId.html#encrypted_param/1","title":"EctoEncryptedId.encrypted_param/1","type":"function"},{"doc":"In your schema module:\n\n    defimpl Phoenix.Param, for: __MODULE__ do\n      defdelegate to_param(term), to: EctoEncryptedId, as: :encrypted_param\n    end","ref":"EctoEncryptedId.html#encrypted_param/1-example","title":"Example - EctoEncryptedId.encrypted_param/1","type":"function"},{"doc":"Helper function to delegate to Phoenix.Param implementations for Ecto schemas.\nReturns plain version of the id.","ref":"EctoEncryptedId.html#plain_param/1","title":"EctoEncryptedId.plain_param/1","type":"function"},{"doc":"In your schema module:\n\n    defimpl Phoenix.Param, for: __MODULE__ do\n      defdelegate to_param(term), to: EctoEncryptedId, as: :plain_param\n    end","ref":"EctoEncryptedId.html#plain_param/1-example","title":"Example - EctoEncryptedId.plain_param/1","type":"function"},{"doc":"Module that allows to work with encrypted values directly.","ref":"EctoEncryptedId.Encryption.html","title":"EctoEncryptedId.Encryption","type":"module"},{"doc":"Decode the value from Base62, then decrypt using AES256GCM cypher.","ref":"EctoEncryptedId.Encryption.html#decrypt/2","title":"EctoEncryptedId.Encryption.decrypt/2","type":"function"},{"doc":"Encrypt the value using AES256GCM cypher, then encode using Base62","ref":"EctoEncryptedId.Encryption.html#encrypt/3","title":"EctoEncryptedId.Encryption.encrypt/3","type":"function"},{"doc":"Example module implementing encrypted field. Provides documentation for the\ninjected functions and modules.","ref":"EctoEncryptedId.ExampleField.html","title":"EctoEncryptedId.ExampleField","type":"module"},{"doc":"","ref":"EctoEncryptedId.ExampleField.html#cast/1","title":"EctoEncryptedId.ExampleField.cast/1","type":"function"},{"doc":"","ref":"EctoEncryptedId.ExampleField.html#dump/1","title":"EctoEncryptedId.ExampleField.dump/1","type":"function"},{"doc":"","ref":"EctoEncryptedId.ExampleField.html#embed_as/1","title":"EctoEncryptedId.ExampleField.embed_as/1","type":"function"},{"doc":"","ref":"EctoEncryptedId.ExampleField.html#equal?/2","title":"EctoEncryptedId.ExampleField.equal?/2","type":"function"},{"doc":"","ref":"EctoEncryptedId.ExampleField.html#load/1","title":"EctoEncryptedId.ExampleField.load/1","type":"function"},{"doc":"","ref":"EctoEncryptedId.ExampleField.html#type/0","title":"EctoEncryptedId.ExampleField.type/0","type":"function"},{"doc":"Container type for the encrypted field.\nImplements struct storing the id and related functions.","ref":"EctoEncryptedId.ExampleField.Id.html","title":"EctoEncryptedId.ExampleField.Id","type":"module"},{"doc":"Create id struct from an encrypted value.\n\nSupported options:\n- :secret_key_fn - custom function to fetch secret key\n\nReturns `{:ok, %Id{}}` on success or :error on failure","ref":"EctoEncryptedId.ExampleField.Id.html#from_encrypted/2","title":"EctoEncryptedId.ExampleField.Id.from_encrypted/2","type":"function"},{"doc":"Create id struct from an encrypted value.\n\nSupported options:\n- :secret_key_fn - custom function to fetch secret key\n\nReturns `%Id{}` on success.\nRaises `EctoEncryptedId.DecryptionError` on failure.","ref":"EctoEncryptedId.ExampleField.Id.html#from_encrypted!/2","title":"EctoEncryptedId.ExampleField.Id.from_encrypted!/2","type":"function"},{"doc":"Create id struct from a plain integer value.\n\nSupported options:\n- :secret_key_fn - custom function to fetch secret key\n\nReturns `%Id{}`.","ref":"EctoEncryptedId.ExampleField.Id.html#from_plain/2","title":"EctoEncryptedId.ExampleField.Id.from_plain/2","type":"function"},{"doc":"","ref":"EctoEncryptedId.ExampleField.Id.html#t:t/0","title":"EctoEncryptedId.ExampleField.Id.t/0","type":"type"},{"doc":"Exception raised when bang `from_encrypted!/2` functions can't decrypt a value","ref":"EctoEncryptedId.DecryptionError.html","title":"EctoEncryptedId.DecryptionError","type":"exception"},{"doc":"# About\n\nThe library provides an easy way to create deterministically encrypted \nand authenticated integer id fields for Ecto schemas. The fields can be \nused to hide actual integer PK values from the outside world by replacing\nthem with encrypted versions (for example, in URLs).","ref":"readme.html","title":"About","type":"extras"},{"doc":"* Thanks to the library it's possible to prevent resource enumeration\nattacks. See https://owasp.org/www-community/attacks/Forced_browsing\n\n* Also, we may want to use URL as a secret to access a resource.\nYes, it's an example of \"security by obscurity\", but sometimes it's the\nright solution.\n\n* We may want to hide some business information. If I'm registering on \nyour fresh website, and my user id is 5 - as a user I will instantly know\nI'm one of the early adopters and your business is not as established\nas it portrays itself!\n\nUsing encryption on-the-fly is a great alternative to using large primary\nkeys (such as UUID) in your database - in many cases it's much better \nto spend CPU cycles of your application servers rather than storage, RAM,\nand CPU of the DB server.\n\nThe library mostly targets Postgres and you can encrypt any integer \ncolumn Postgres supports (normal and autoincrement, signed and unsigned).\n\nWhen using the library resource URLs may look like this:\nhttps://example.com/posts/GLWsqG8DwIUxd7MecoTzDPg0fSLDN74qEyYy9Dw82SInd77vSi2Ops","ref":"readme.html#why","title":"Why - About","type":"extras"},{"doc":"The package can be installed by adding `ecto_encrypted_id` to your list of\ndependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ecto_encrypted_id, \"~> 0.1\"}\n  ]\nend\n```\nPrepare a 32bytes encryption key. If you are using Phoenix:\n```shell\n$ mix phx.gen.secret 32\n\n```\nBy default the library will try to use secret key provided in the application\nenvironment. You can configure it in `runtime.exs` like this, for example:\n```elixir\nconfig :ecto_encrypted_id, :settings,\n  secret_key: System.fetch_env!(\"SECRET\")\n```\nIf you don't like to use application environment, you can inject a function\nproviding the key (we will discuss it later).","ref":"readme.html#installation","title":"Installation - About","type":"extras"},{"doc":"For every Ecto schema that will use an encrypted id in your project you should \nconsider creating a separate module. The name of the module doesn't matter, but\nit's better to name your modules in a way that it's easy to understand which Id module \ncorresponds to which schema. Let's say you have a Post schema:\n```elixir\ndefmodule MyProject.PostId do\n  use EctoEncryptedId, salt: \"my salt\"\nend\n```\nNote the `salt` parameter: it must be a string that is unique between field modules,\nso that different models (schemas) don't share the same encrypted ids.\n\nIf you prefer to provide secret key in a different way, not by using app \nenvironment, you can pass a function returning the key like this:\n```elixir\nuse EctoEncryptedId, salt: \"my salt\", secret_key_fn: &MyProject.Secret.key/0\n```\n\nAfter that, add the field as a primary key for the schema. This example\nis using autoincrementing integer PK:\n```elixir\ndefmodule MyProject.Post do\n  use Ecto.Schema\n\n  @primary_key {:id, MyProject.PostId, autogenerate: true}\n  schema \"posts\" do\n    field :title, :string\n  end\nend\n```\nNow, every time you load the schema instance from the DB, `id` field\nwill contain an instance of `MyProject.PostId.Id`. See the \n`EctoEncryptedId.ExampleField.Id` module docs to get an idea on how to use it.\n\nAlso, if you are using Phoenix, consider adding `Phoenix.Param` implementation,\nto be able to use your id in URLs, same way like you could with integers.\nIf you want to use encrypted versions:\n```elixir\ndefmodule MyProject.Post do\n  ...\n  defimpl Phoenix.Param, for: __MODULE__ do\n    defdelegate to_param(term), to: EctoEncryptedId, as: :encrypted_param\n  end\n  ...\nend\n```\nOr if you want to keep the unencryped versions (not that it makes much sense,\nbut anyway):\n```elixir\ndefmodule MyProject.Post do\n  ...\n  defimpl Phoenix.Param, for: __MODULE__ do\n    defdelegate to_param(term), to: EctoEncryptedId, as: :plain_param\n  end\n  ...\nend\n```\nThe library provides `Phoenix.HTML.Safe` implementation that outputs encrypted \nversions in HTML automatically. This way you won't accidentally leak plain text\nids.","ref":"readme.html#usage","title":"Usage - About","type":"extras"},{"doc":"It worth to note that using a solution like this doesn't come without costs.\nIt may take a bit of time getting used to non-scalar id in models. The additional \ndecryption/encryption step for URLs may be annoying, so you have to decide if the \nadvantages are worth the trouble in your case. The library doesn't try to make \nimportant decisions implicitly - as a developer you will have to decide whether \nto use encrypted version or integer one, on a case-by-case basis.\n\nThe encryption we use is deterministic - the encrypted id is defined by the secret\nkey and the salt. As long as you don't change them you can use the encrypted ids in\npermalinks.\n\nAlso, as we discussed earlier, every field module should use different salt. The salt\nis not secret, but the encryption key is.\n\nDon't use existing secret keys from your project as the key for this library -\nin case there's been a possible leak, you should always change secrets that guard\nreal security stuff, like passwords and bank accounts, but in the case of this \nlibrary, you could even prefer to keep using the leaked key just to avoid breaking\nthe permalinks. It's not like we are guarding government secrets here, right ? RIGHT ?\n\nThe encryption is intended to be strong enough for most cases (unless the attacker\nis a 3-letter agency). But if you know something about cryptography your feedback\nwould be welcome.","ref":"readme.html#important-considerations","title":"Important Considerations - About","type":"extras"}],"producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}